package common;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.WritableByteChannel;

public class BufferedWritableByteChannel implements WritableByteChannel {
	//private File file;
	private RandomAccessFile randomAccessFile;
	private FileChannel fileChannel;
	private long filePosition;
	private final int size;
	private MappedByteBuffer buffer;
	private MappedByteBuffer lastBuffer;
	
	public BufferedWritableByteChannel(File file, int size) throws IOException {
		//this.file=file;
		randomAccessFile=new RandomAccessFile(file,"rw");
		fileChannel=randomAccessFile.getChannel();
		this.size=size;
		buffer=DirectMemoryManager.mapMemory(randomAccessFile.getChannel(),FileChannel.MapMode.READ_WRITE, 0, size);
	}
	
	public int write(ByteBuffer src) throws IOException {
		//bufferList.add(src);
		if (buffer.remaining()> src.remaining()) {
			buffer.put(src);
			if (buffer.remaining()==0) {
				mapNextChunk();
			}
		} else {
			//System.out.println("Remaining: " + buffer.remaining());
			int limit=src.limit();
			src.limit(src.position() + buffer.remaining());
			buffer.put(src);
			src.limit(limit);
			ByteBuffer remainder=src.slice();
			mapNextChunk();
			buffer.put(remainder);
			//System.gc();
		}
		return src.limit();
	}

	private void mapNextChunk() throws IOException {
		filePosition+=buffer.capacity();
		if (lastBuffer != null) {
			lastBuffer.force();
		}
		lastBuffer=buffer;
		//new MappedBufferForcer(buffer);
		buffer=DirectMemoryManager.mapMemory(fileChannel, FileChannel.MapMode.READ_WRITE, filePosition, size);
	}
	public void close() throws IOException {
		lastBuffer=null;
		if (buffer != null) {
			flush();
			long finalLen=filePosition+buffer.position();
			buffer=null;
			while (true) {
				System.gc();
				Thread.yield();
				try {
					fileChannel.truncate(finalLen);
					break;
				} catch (IOException ioe) {
				}
			}
			randomAccessFile.close();
		}
	}
	private void flush() throws IOException {
		buffer.limit(buffer.position());
		buffer.force();
	}
	public boolean isOpen() {
		return fileChannel.isOpen();
	}

}
