package common;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

public class MultiFileInputStream extends InputStream {
	private Logger logger=Logger.getLogger(this.getClass());
	private int index=-1;
	private File files[];
	private InputStream in;
	
	
	class NullInputStream extends InputStream {
		public int read() {
			return -1;
		}
	}
	
	public MultiFileInputStream(File files[]) throws FileNotFoundException {
		this.files=files;
		in=getNextInputStream();
	}

	public int read() throws IOException {
		int myByte=in.read();
		if (myByte<0) {
			if (!((in=getNextInputStream()) instanceof NullInputStream)) {
				read();
			}
		}
		return myByte;
	}
	
	public int read(byte buffer[]) throws IOException {
		return read(buffer, 0, buffer.length);
	}
	
	public int read(byte buffer[], int offset, int len) throws IOException {
		int myBytes=in.read(buffer, offset, len);
		if (myBytes != len) {
			if (!((in=getNextInputStream()) instanceof NullInputStream )) {
				if (myBytes==-1) {
					myBytes=0;
				}
				//Possible bug if next file returns -1;
				myBytes += read(buffer, offset + myBytes, len-myBytes);
			} else {
				return myBytes;
			}
		}
		return myBytes;
	}

	public int available() throws IOException {
		return in.available();
	}
	
	public long skip(long l) throws IOException {
		long totalBytes=0;
		while (in != null && l > 0) {
			totalBytes+=in.skip(l);
			if (l!=totalBytes) {
				in=getNextInputStream();
			}
		}
		return totalBytes; 
	}
	
	private InputStream getNextInputStream() throws FileNotFoundException {
		++index;
		if (in != null) {
			try {
				in.close();
			} catch (IOException e) {
			}
		}
		if (index < files.length) {
			logger.info("Switching Files: " + files[index]);
			return new FileInputStream(files[index]);
		} else {
			return new NullInputStream();
		}
	}
	public void close() throws IOException {
		in.close();
	}

}
