package common;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.filechooser.FileSystemView;

public class Util {
	
	private static final char hex[]={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
	private static final String hexLower="0123456789abcdef";
	
	//TIVO_TIMESCALE 1000000000 
	public static final long TIVO_TIMESCALE_MILLIS=1000000;
	
	public static void appendHex(byte myByte, StringBuffer sb) {
		sb.append(hex[(myByte&0xF0)>>4]);
		sb.append(hex[myByte&0xF]);
	}
	public static final void printHex(byte myByte) {
		System.out.print(hex[(myByte&0xF0)>>4]);
		System.out.print(hex[myByte&0xF]);
		System.out.print(' ');
	}
	public static void printHex(byte packet[], int offset, int size) {
		StringBuffer sb=new StringBuffer(size*2);
		//out.write(packet, offset, size);
		for (int i=0;i < size;i++) {
			appendHex(packet[offset+i],sb);
			sb.append(' ');
		}
		System.out.print(sb);
		//System.out.println();
	}
	/**
	 * Takes a hex string and converts it to a byte array
	 * @param hex xx xx xx xx
	 * @return
	 */
	public static byte[] hexToBytes(String hex) {
		hex=hex.toLowerCase();
		int byteCount=(hex.length() + 1)/3;
		byte bytes[]=new byte[byteCount];
		for (int i=0;i<byteCount;i++) {
			bytes[i]=(byte)Integer.parseInt(hex.substring(i * 3, i * 3 + 2), 16);
		}
		return bytes;
	}
	public static void longToHex(long number, int digits, StringBuffer stringBuffer) {
		int len=stringBuffer.length();
		stringBuffer.append("0000000000000000".substring(0, digits));
		for (int i=digits-1;i>=0;i--) {
			int hex=(int)(number & 0xF);
			stringBuffer.setCharAt(len+i, hexLower.charAt(hex));
			number >>>= 4;
		}
	}
	public static File getRoot(File file) {
		File root=file;
		while (root.getParentFile() !=null) {
			root=root.getParentFile();
		}
		return root;
	}
	
	public static String getVolumeLabel(File file) {
		FileSystemView view = FileSystemView.getFileSystemView();
		String name = view.getSystemDisplayName(getRoot(file));
		if (name == null) { return null; }
		name = name.trim();
		if (name == null || name.length() < 1) {
			return null;
		}
		/*int index = name.lastIndexOf(" (");
		if (index > 0) {
			name = name.substring(0, index);
		}*/
		return name;
	}
	/**
	 * @param packet
	 * @param offset
	 * @param number
	 */
	public static void set16(byte packet[], int offset, int number) {
		packet[offset]=(byte)((number)>>8);
		packet[offset+1]=(byte)((number)&0xFF);
	}
	public static void set32(byte packet[], int offset, long number) {
		packet[offset]=(byte)((number)>>24);
		packet[offset+1]=(byte)((number)>>16);
		packet[offset+2]=(byte)((number)>>8);
		packet[offset+3]=(byte)((number)&0xFF);
	}
	
	/**
	 * Int is actually 32 bit, but using long avoids the sign problem
	 * @param packet
	 * @param offset
	 * @return
	 */
	public static long get32(byte packet[], int offset) {
		return getLong(packet, offset, 4, false);
	}
	/**
	 * Lets hope we don't get anything with 64 bits, or we'll have a negative number
	 * @param packet
	 * @param offset
	 * @return
	 */
	public static long get64(byte packet[], int offset) {
		return getLong(packet, offset, 8, false);
	}
	public static long getLong(ByteBuffer buffer, int offset, int size, boolean littleEndian) {
		long l=0;
		int i, end, dir;
		if (littleEndian) {
			i=size-1;
			end=-1;
			dir=-1;
		} else {
			i=0;
			end=size;
			dir=1;
		}
		for (;i!=end;i+=dir) {
			l <<=8;
			l +=(buffer.get(offset + i)&0xFF);
		}
		return l;
	}
	public static long getLong(byte buffer[], int offset, int size, boolean littleEndian) {
		long l=0;
		int i, end, dir;
		if (littleEndian) {
			i=size-1;
			end=-1;
			dir=-1;
		} else {
			i=0;
			end=size;
			dir=1;
		}
		for (;i!=end;i+=dir) {
			l <<=8;
			l +=(buffer[offset + i]&0xFF);
		}
		return l;
	}
	public static boolean validateTime(long timeStamp, long lastTimeStamp, long deltaPerChunk) {
		if (deltaPerChunk==0) {
			return true;
		}
		long delta=timeStamp-lastTimeStamp;
		return (delta < deltaPerChunk);
	}
	
	public static String timeToString(long totalMillis) {
		String sign="";
		if (totalMillis < 0) {
			sign="-";
			totalMillis*=-1;
		}
		long millis=totalMillis%1000;
		long totalSecs=totalMillis/1000;
		long secs=totalSecs%60;
		long totalMinutes = totalSecs/60;
		long minutes=totalMinutes % 60;
		long hours=totalMinutes/60;
		String millisString=Long.toString(millis);
		millisString = ".00".substring(0, 4-millisString.length()) + millisString;
		return sign + hours + ':' + (minutes < 10 ? "0" : "") + minutes  + ':' + (secs < 10 ? "0" : "") + secs + millisString;
	}
	public static String tivoTimeToString(long tivoTime) {
		long totalMillis=tivoTime/TIVO_TIMESCALE_MILLIS;
		return timeToString(totalMillis);
	}
	public static InetAddress getLocalAddress() throws UnknownHostException {
		List list=getInetAddresses(false);
		if (list.size() >0) {
			return (InetAddress)list.get(0);
		} else {
			return null;
		}
	}
	public static List getInetAddresses(boolean includeLoopback) {
		ArrayList list=new ArrayList(3);
		try {
			for (Enumeration nie=NetworkInterface.getNetworkInterfaces();nie.hasMoreElements();) {
				NetworkInterface ni=(NetworkInterface)nie.nextElement();
				for (Enumeration iae=ni.getInetAddresses();iae.hasMoreElements();) {
					
					InetAddress ie=(InetAddress)iae.nextElement();
					if ((includeLoopback || !ie.isLoopbackAddress())&& !list.contains(ie) ) {
						list.add(ie);
					}
				}
			}
		} catch (SocketException e) {
			e.printStackTrace();
		}
		
		return list;
	}
	public static String generateUuid() {
		StringBuffer uuid=new StringBuffer(50);
		//uuid.append("uuid:");
		List addresses=getInetAddresses(false);
		InetAddress inetAddress=(InetAddress)addresses.get(0);
		byte rawAddress[]=inetAddress.getAddress();
		long lead=0;
		for (int i=0;i<rawAddress.length;i++) {
			lead <<=8;
			lead +=(0xFF)&rawAddress[i];
		}
		//long lead=(long)(Math.random() * (256d*256d*256d*256d));
		longToHex(lead, 8, uuid);
		uuid.append("-1234-5678-9abc-");
		longToHex(56178, 12, uuid);
		return uuid.toString();

	}
	public static void main(String args[]) {
		byte buffer[]={2,1,0,0};
		System.out.println(getLong(buffer,0,4,true));
		System.out.println(getLong(buffer,0,4,false));
	}
	public static String replace(final String aInput, final String aOldPattern, final String aNewPattern){
		if ( aOldPattern.equals("") ) {
			throw new IllegalArgumentException("Old pattern must have content.");
		}
	
		final StringBuffer result = new StringBuffer();
		//startIdx and idxOld delimit various chunks of aInput; these
		//chunks always end where aOldPattern begins
		int startIdx = 0;
		int idxOld = 0;
		
		while ((idxOld = aInput.indexOf(aOldPattern, startIdx)) >= 0) {
			//grab a part of aInput which does not include aOldPattern
			result.append( aInput.substring(startIdx, idxOld) );
			//add aNewPattern to take place of aOldPattern
			result.append( aNewPattern );
	
			//reset the startIdx to just after the current match, to see
			//if there are any further matches
			startIdx = idxOld + aOldPattern.length();
		 }
		//the final chunk will go to the end of aInput
		result.append( aInput.substring(startIdx) );
		return result.toString();
	}
	public static int readFully(final byte buffer[], final InputStream in) throws IOException {
		return readFully(buffer, in, buffer.length);
	}
	
	public static int readFully(final byte buffer[], final InputStream in, final int bytesToRead) throws IOException {
		int totalBytes=in.read(buffer,0,bytesToRead);
		while (totalBytes !=bytesToRead) {
			//Kind of squirelly logic, but we want to get the default path the fastest
			if (totalBytes==-1) {
				break;
			}
			int tmp=in.read(buffer, totalBytes, bytesToRead - totalBytes);
			if (tmp==-1) {
				break;
			}
			totalBytes+=tmp;
		}
		return totalBytes;
	}
	
	public static int parseNumber(String text) {
		if (text.startsWith("0x")) {
			text=text.toLowerCase();
			int value=0;
			for (int i=2;i<text.length();i++) {
				value<<=4;
				int digit=hexLower.indexOf(text.charAt(i));
				if (digit < 0) {
					throw new NumberFormatException("Invalid hex digit: " + text.charAt(i));
				} else {
					value +=digit;
				}
			}
			return value;
		} else {
			return Integer.parseInt(text);
		}
	}
	public static int get16(byte packet[], int offset) {
		return  ((packet[offset]&0xFF)<<8) + (packet[offset+1]&0xFF);
	}
	public static int findBytes(ByteBuffer buffer, byte bytes[], int offset) {
		final byte firstByte=bytes[0];
		final int end=buffer.limit()-bytes.length;
		
		for (int i=offset; i<end;i++) {
			if (buffer.get(i)==firstByte) {
				int j=0;
				while (j<bytes.length) {
					if (buffer.get(i+j)==bytes[j]) {
						j++;
					} else {
						break;
					}
				}
				if (j==bytes.length) {
					return i;
				}
			}
		}
		return -1;
	}

}
