package common.ts;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import common.DirectMemoryManager;
import common.StreamDescriptor;
import common.mpeg.MpegUtil;

import demux.ts.TsInfo;
import demux.ts.TsRawPacketPart;

public abstract class TsTable implements TsConstants {
	
	protected int pid;
	private byte tableId;
	protected ByteBuffer out;
	protected boolean found;
	protected boolean required=true;
	protected TsRawPacketPart buffer[];
	protected boolean complete;
	private ArrayList<TsTableParsedListener> listenerList=new ArrayList<TsTableParsedListener>(2);

	/**
	 * Variable used to echo the input if no processing is done
	 */
	protected final TsRawPacketPart[] COPY=new TsRawPacketPart[1];
	
	protected abstract TsRawPacketPart[] parse();
	
	protected abstract TsRawPacketPart[] filter(boolean reparse);
		
	protected TsTable(int pid, byte tableId) {
		this.pid=pid;
		this.tableId=tableId;
	}
	
	protected TsRawPacketPart[] mergeOut(TsRawPacketPart buffer[]) {
		int i=0;
		out.position(0);
		ByteBuffer myBuffer=null;
		for (int len=out.limit();len>0;i++) {
			myBuffer=buffer[i].getByteBuffer();
			int syncOffset=buffer[i].getTsSyncOffset();
			if (myBuffer.isReadOnly()) {
				myBuffer.position(0);
				ByteBuffer newBuffer=DirectMemoryManager.allocateDirect(myBuffer.capacity());
				copy(myBuffer,newBuffer,syncOffset+3);
				//Clear the adaption field flag as we remove these
				int b=myBuffer.get() & 0xdf;
				newBuffer.put((byte)b);
				buffer[i].setByteBuffer(myBuffer=newBuffer);
			} else {
				myBuffer.position(syncOffset+4);
			}
			int bytes=Math.min(myBuffer.remaining(), len);
			copy(out,myBuffer,bytes);
			//out.limit(out.position()+myBuffer.remaining());
			//myBuffer.put(out);
			len-=bytes;
		}
		while (myBuffer.remaining() > 0) {
			myBuffer.put((byte)0xff);
		}
		TsRawPacketPart myOut[]=new TsRawPacketPart[i];
		for (int j=0;j<i;j++) {
			myOut[j]=buffer[j];
		}
		return myOut;
	}

	/*public TsRawPacketPart[] createFilterOut(int pid) {
		int packets=out.limit()/184 + 1;
		TsRawPacketPart buffer[]=new TsRawPacketPart[packets];
		for (int i=0;i<packets;i++) {
			buffer[i]=new TsRawPacketPart(TS_PACKET_SIZE);
			ByteBuffer byteBuffer=buffer[i].getByteBuffer();
			byteBuffer.put(TS_SYNC_BYTE);
			short pidOut=(short)(0x400 + pid);
			byteBuffer.putShort(pidOut);
			byteBuffer.put((byte)(0x10 + i));
		}
		return filter(buffer,false);
	}*/
	protected ByteBuffer mergeIn(TsRawPacketPart buffer[]) {
		int len=getLen(buffer);
		ByteBuffer tmp=ByteBuffer.allocate(len);
		for (int i=0; len > 0;i++) {
			ByteBuffer myBuffer=buffer[i].getByteBuffer();
			int offset=(i==0)?getTableStart(buffer):(buffer[i].getTsSyncOffset() + 4);
			myBuffer.position(offset);
			myBuffer.limit(Math.min(myBuffer.capacity(), offset+len));
			tmp.put(myBuffer);
			myBuffer.limit(myBuffer.capacity());
			//copy(myBuffer, tmp, Math.min(myBuffer.capacity(), syncOffset+4+len));
			len -=(buffer[0].limit()-offset);
		}
		tmp.flip();
		return tmp;
	}
	public static final int getLen(TsRawPacketPart buffer[]) {
		
		return (buffer[0].getByteBuffer().getShort(getTableStart(buffer)+2) & 0xfff)+4;
	}
	public static final int getTableStart(TsRawPacketPart buffer[]) {
		int offset=buffer[0].getTsSyncOffset() + 4;
		byte tableOffset=buffer[0].get(offset);
		if (tableOffset != 0) {
			offset+=(tableOffset & 0xff) + 1;//extra 1 of the offset byte;
		} 
		return offset;
	}
	protected void copy(final ByteBuffer source, final ByteBuffer target, int count) {
		while (count > 7) {
			target.putLong(source.getLong());
			count-=8;
		}
		while (count > 0) {
			target.put(source.get());
			count--;
		}
	}

	protected void finalizeOut() {
		out.putShort(2, (short)(out.position() | (out.getShort(2) & 0xf000))); //len=position() is actually more of a coincidence because you need add 4 and subtract 4
		int crc=MpegUtil.calculateCRC32(out, 1, out.position());
		out.putInt(crc);
		out.flip();		
	}
	public final byte getTableId() {
		return tableId;
	}
	public final int getPid() {
		return pid;
	}
	public final boolean isFound() {
		return found;
	}
	public final boolean isRequired() {
		return required;
	}
	public final boolean isReady() {
		if (required) {
			return found;
		} else {
			return true;
		}
	}
	//TODO: Refactor code to call these instead of having TsRawDemuxer do it directly
	public void setStreamList(List<StreamDescriptor> list) {
		//Implement me!
	};
	public void setSelectedProgramNumbers(short programNumbers[], TableHandler tableHandler) {
		//Implement me!
	};
	/**
	 * 
	 * @param part
	 * @return true if the buffer is complete and the tableId we are watching
	 */
	public boolean append(TsRawPacketPart part) {
		int bytes=184;
		TsInfo tsInfo=part.getTsInfo();
		if (tsInfo.pidStart) {
			COPY[0]=part;
			buffer=COPY;
		} else if (buffer==null || complete) {
			//This means we started in the middle of a table
			//COPY[0]=part;
			return false;
		} else {
			TsRawPacketPart tmp[]=new TsRawPacketPart[buffer.length+1];
			int i=0;
			while (i<buffer.length) {
				tmp[i]=buffer[i];
				i++;
				bytes+=184;
			}
			tmp[i]=part;
			buffer=tmp;
		}
		//Is that the end?
		complete=bytes >= TsTable.getLen(buffer);
		if (complete) {
			int offset=buffer[0].getTsSyncOffset();

			byte tableOffsetByte=buffer[0].get(offset+4);
			int tableOffset=tableOffsetByte==0?0:((tableOffsetByte &0xff)+1);
			byte tableId=buffer[0].get(offset+5+tableOffset);
			
			return tableId==this.tableId;
		}
		return false;
	}
	
	public void addListener(TsTableParsedListener listener) {
		listenerList.add(listener);
	}
	protected void notifyListeners() {
		TsRawPacketPart tmpBuffer[]=new TsRawPacketPart[buffer.length];
		System.arraycopy(buffer, 0, tmpBuffer, 0, buffer.length);
		for (int j=0;j<listenerList.size();j++) {
			listenerList.get(j).tsTableDidParse(this, tmpBuffer);
		}		
	}
}
