package demux.pes;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.TimeZone;
import java.util.TreeMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import common.DiskStream;
import common.StreamDescriptor;
import common.Util;

public class XplHelper {
	public static final String HD_HDVD_PLAYLIST_EXT=".xpl";
	private static final String DISK_PATH="file:///dvddisc";
	private static final String MAIN_TITLE_IDS="|MainMovie|mainMovie|feature|Feature|Film|Title001|";
	//private static final String MAIN_MOVIE="MainMovie";
	//private static final String FEATURE="feature";
	private static final String TITLE="Title";
	private static final String TITLE_DURATION="titleDuration";
	private static final String PRIMARY_AUDIO_VIDEO_CLIP="PrimaryAudioVideoClip";
	private static final String CHAPTER="Chapter";
	private SimpleDateFormat formatter=new SimpleDateFormat("HH:mm:ss:SSS");
	private static final String INTERPOL_EVO="interpol.evo";
	
	private Document dom;
	private File file;
	
	public XplHelper(File file) throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		//Using factory get an instance of document builder
		DocumentBuilder db = dbf.newDocumentBuilder();
		this.file=file;
		//parse using builder to get DOM representation of the XML file
		dom = db.parse(file);
		
		formatter.setTimeZone(TimeZone.getTimeZone("GMT"));

	}
	
	private Element findAudioVideoClipByFileName(File file) {
		String fileName=file.getName().toUpperCase();
		String mapFileName=DISK_PATH+"/HVDVD_TS/" + fileName.substring(0, fileName.lastIndexOf('.')) + ".MAP";
		
		NodeList titleList=dom.getElementsByTagName(TITLE);
		for (int t=0;t<titleList.getLength();t++) {
			Element title=(Element)titleList.item(t);
			NodeList avClipList=title.getElementsByTagName(PRIMARY_AUDIO_VIDEO_CLIP);
			for (int c=0;c<avClipList.getLength();c++) {
				Element avClip=(Element)avClipList.item(c);
				String src=avClip.getAttribute("src");
				if (src.equals(mapFileName)) {
					return avClip;
				}
			}
		}
		return null;
	}
	
	public DiskStream findMainMovie() {
		NodeList titleList=dom.getElementsByTagName(TITLE);
		for (int t=0;t<titleList.getLength();t++) {
			Element title=(Element)titleList.item(t);
			String id=title.getAttribute("id");
			if (MAIN_TITLE_IDS.indexOf('|'+id+'|') >=0) {
				String titleDuration=title.getAttribute(TITLE_DURATION);
				long titleTime=hdDvdTimeToMillis(titleDuration);
				NodeList avClipList=title.getElementsByTagName(PRIMARY_AUDIO_VIDEO_CLIP);
				
				ArrayList<File> fileList=new ArrayList<File>(avClipList.getLength());
				for (int c=0;c<avClipList.getLength();c++) {
					Element avClip=(Element)avClipList.item(c);
					String src=avClip.getAttribute("src");
					String evoSrc=src.substring(DISK_PATH.length(), src.length()-3) + "EVO";
					if (!evoSrc.toLowerCase().endsWith(INTERPOL_EVO)) {
						fileList.add(new File(Util.getRoot(file), evoSrc));
					}
				}
				File fileArray[]=new File[fileList.size()];
				fileList.toArray(fileArray);

				NodeList chapterList=title.getElementsByTagName(CHAPTER);
				TreeMap<Long, String> chapters=new TreeMap<Long, String>();
				for (int c=0;c<chapterList.getLength();c++) {
					Element chapter=(Element)chapterList.item(c);
					String begin=chapter.getAttribute("titleTimeBegin");
					String name=chapter.getAttribute("displayName");
					chapters.put(hdDvdTimeToMillis(begin),name);
				}

				return new DiskStream(fileArray, titleTime, file, chapters);
			}
		}
		return null;
	}
	private long hdDvdTimeToMillis(String hdDvdTime) {
		long titleTime=0;
		try {
			titleTime=formatter.parse(hdDvdTime+"0").getTime();
			long subSeconds=(titleTime%1000);
			titleTime-=subSeconds;
			long millis=(subSeconds * 100)/60;
			titleTime+=millis;
		} catch (ParseException e) {
		}		
		return titleTime;
	}
	public void setLanguage(File file, Map<Integer,StreamDescriptor> streamDescriptorMap) {
		Element avClip=findAudioVideoClipByFileName(file);
		if (avClip !=null) {
			for (Iterator<StreamDescriptor> i=streamDescriptorMap.values().iterator();i.hasNext();) {
				StreamDescriptor sd=i.next();
				int id=sd.getStreamId() & 0xf;
				String typeName=null;
				String idString=null;
				if (sd.getClassification()==StreamDescriptor.CLASSIFICATION_AUDIO) {
					if (id < 8) {
						typeName="Audio";
						idString=Integer.toString(id+1);
					} else {
						typeName="SubAudio";
						idString=Integer.toString((id&0x7)+1);
					}
				} else if (sd.getClassification()==StreamDescriptor.CLASSIFICATION_SUBTITLE) {
					typeName="Subtitle";
					idString=Integer.toString(id+1);
				} else if (sd.getClassification()==StreamDescriptor.CLASSIFICATION_VIDEO && id > 0x5) {
					typeName="SubVideo";
					idString=Integer.toString(id-5);
				}
				if (typeName !=null) {
					NodeList typeNodeList=avClip.getElementsByTagName(typeName);
					for (int t=0;t<typeNodeList.getLength();t++) {
						Element type=(Element)typeNodeList.item(t);
						String track=type.getAttribute("track");
						if (track.equals(idString)) {
							String description=type.getAttribute("description");
							if (description.length() > 0) {
								sd.setLanguage(description);
							}
						}
					}
				}
			}
		}
	}
}
