package demux.ts;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.WritableByteChannel;



public class M2tsRawPacketPart extends TsRawPacketPart {
	public static final int ARRIVAL_TIME_MASK=0x3fffffff;
	private static final int DEFAULT_ARRIVAL_TIME=-1;
	private static final ByteBuffer arrivalTimeBuffer=ByteBuffer.allocateDirect(4);
	private int arrivalTime;
	
	private final int tsSyncOffset;
	
	public M2tsRawPacketPart(int len, int tsSyncOffset) {
		super(len);
		this.tsSyncOffset=tsSyncOffset;
	}
	public M2tsRawPacketPart(ByteBuffer buffer, int tsSyncOffset) {
		super(buffer);
		this.tsSyncOffset=tsSyncOffset;
	}	
	public int getTsSyncOffset() {
		return tsSyncOffset;
	}

	public void offsetClock(long clockOffset) {
		super.offsetClock(clockOffset);
		
		arrivalTime=(buffer.getInt(0) + (int)(clockOffset * 300)) & ARRIVAL_TIME_MASK;
	}
	
	public void setArrivalTime(int arrivalTime) {
		this.arrivalTime=arrivalTime;
	}

	@Override
	public void setByteBuffer(ByteBuffer byteBuffer) {
		super.setByteBuffer(buffer);
		arrivalTime=DEFAULT_ARRIVAL_TIME;
	}
	@Override
	public int write(WritableByteChannel out) throws IOException {
		if (arrivalTime==DEFAULT_ARRIVAL_TIME) {
			return super.write(out);
		} else {
			int bytes;
			synchronized(arrivalTimeBuffer) {
				arrivalTimeBuffer.putInt(0, arrivalTime);
				arrivalTimeBuffer.position(0);
				bytes=out.write(arrivalTimeBuffer);
			}
			buffer.position(tsSyncOffset);
			return out.write(buffer) + bytes;
		}
	}
	
}
