package muxer.demux;

import java.io.File;
import java.io.IOException;
import java.nio.channels.WritableByteChannel;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import muxer.MuxFilter;

import common.BufferedWritableByteChannel;
import common.Logger;
import common.StreamDescriptor;
import common.mpeg.MpegUtil;
import common.packet.ClockReferenceI;
import common.packet.Packet;

public class RawDemuxerMuxer extends MuxFilter {
	private Logger logger=Logger.getLogger(this.getClass());
	private java.util.HashMap<Integer, StreamData> streamOutMap=new HashMap<Integer, StreamData>();
	
	protected String fileNamePrefix;
	
	protected class StreamData {
		protected String fileName;
		protected WritableByteChannel out;
		protected long earliestPts=Long.MAX_VALUE;
		protected int count;
		protected int classification;
		
		protected StreamData(String fileName, int classification) throws IOException {
			this.fileName=fileName;
			//this.out=new FileOutputStream(fileName).getChannel();
			this.out=new BufferedWritableByteChannel(new File(fileName),5*1024*1024);
			this.classification=classification;
		}
		protected void setPts(Packet packet) {
			if (count < 8) {
				count++;
				long pts=packet.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS);
				if (pts >=0 && pts < earliestPts) {
					earliestPts=pts;
				}
			}
		}
	}
	
	public RawDemuxerMuxer(String fileNamePrefix) {
		this.fileNamePrefix=fileNamePrefix;
	}
	protected String getFileName(StreamDescriptor streamDescriptor) {
		return fileNamePrefix + '_' + Integer.toHexString(streamDescriptor.getStreamId())+streamDescriptor.getFileNameExtension();
	}
	/*protected void init() throws IOException {
		Map muxedStreamDescriptorMap=muxManager.getMuxedStreamDescriptorMap();
	}*/	
	public int addPacket(Packet packet) throws IOException {
		Integer streamId=Integer.valueOf(packet.getStreamId());
		StreamData data=(StreamData)streamOutMap.get(streamId);
		if (data==null) {
			Map<Integer, StreamDescriptor> muxedStreamDescriptorMap=muxManager.getMuxedStreamDescriptorMap();
			StreamDescriptor streamDescriptor=muxedStreamDescriptorMap.get(streamId);
			String fileName=getFileName(streamDescriptor);
			logger.debug("Created: " + fileName);
			streamOutMap.put(streamId, data=new StreamData(fileName,streamDescriptor.getClassification()));
		}
		try {
			data.setPts(packet);
			write(packet, data);
		} catch (IOException ioe) {
			close();
			throw ioe;
		}
		return super.addPacket(packet);
	}	
	protected void write(Packet packet, StreamData data) throws IOException {
		packet.getPayload().write(data.out);
	}
	private void close() {
		for (Iterator<StreamData> i=streamOutMap.values().iterator();i.hasNext();) {
			StreamData data=i.next();
			try {
				data.out.close();
			} catch (IOException e) {
			}
		}
	}
	public void streamEnd() {
		close();
		long videoPts=Long.MAX_VALUE;
		Integer videoStreamId=null;
		for (Iterator<Integer> i=streamOutMap.keySet().iterator();i.hasNext();) {
			Integer streamId=i.next();
			StreamData streamData=(StreamData)streamOutMap.get(streamId);
			if (streamData.classification==StreamDescriptor.CLASSIFICATION_VIDEO) {
				videoPts=streamData.earliestPts;
				videoStreamId=streamId;
				break;
			}
		}
		if (videoStreamId !=null) {
			TreeMap<Integer, StreamData> map=new TreeMap<Integer, StreamData>(streamOutMap);
			map.remove(videoStreamId);
			for (StreamData streamData : map.values()) {
				String fileName=streamData.fileName;
				int extIndex=fileName.lastIndexOf('.');
				String newFileName=fileName.substring(0, extIndex) + "-DELAY " + 
					MpegUtil.pesTimeToMillis(streamData.earliestPts-videoPts) + "ms" +
					fileName.substring(extIndex);
				if (new File(fileName).renameTo(new File(newFileName))) {
					fileName=newFileName;
				}
				logger.info(fileName +  " - Offset: " + MpegUtil.pesTimeToString(streamData.earliestPts-videoPts));
			}
		}
	}
	
	public String getFileName(int streamId) {
		StreamData data=(StreamData)streamOutMap.get(Integer.valueOf(streamId));
		return data.fileName;
	}
	/**
	 * 
	 * @param videoStreamId
	 * @param audioStreamId
	 * @return offset in 27mHz/300
	 */
	public long getOffset(int videoStreamId, int audioStreamId) {
		StreamData videoData=(StreamData)streamOutMap.get(Integer.valueOf(videoStreamId));
		StreamData audioData=(StreamData)streamOutMap.get(Integer.valueOf(audioStreamId));
		return audioData.earliestPts-videoData.earliestPts;
	}
	public int getPosition() {
		return POSITION_MUXER;
	}	
}
